//////////////////////////////////////////////////
//
//                      ppa-r
//  point process analysis system: pair correlation
//
//      Copyright  October 2001 by TAKENAKA, A.
//           rev.  November 2004

#include <iostream>

#include "plotarea.h"

//////////////////////////////
//
//  Constructor  # Does nothing.

PlotArea::PlotArea(void)
{}

//////////////////////////////
//

bool PlotArea::loadPoints(const char* file1, const char* file2)
{
    Points1.erase(Points1.begin(), Points1.end());
    Points2.erase(Points2.begin(), Points2.end());

    Points1.load(file1);
    if (Points1.size() == 0) {  //  No data.
        return false;
    }

    if (file2 != NULL) {  // If two files are specified...
        Points2.load(file2);
        if (Points2.size() == 0) {  //  No data.
            return false;
        }
    }

  //  The area of the first data set is stored.

    AreaSize  = Points1.getAreaSize();
    Area      = AreaSize.X * AreaSize.Y;
    Perimeter = 2.0 * (AreaSize.X + AreaSize.Y);

    Density1  = Points1.size() / Area;
    MeanDensity = Density1;

    if (Points2.size() > 0) {
        Density2 = Points2.size() / Area;
        MeanDensity = sqrt(Density1 * Density2);
    }

    if ((Points2.size()) > 0 && !(AreaSize == Points2.getAreaSize()) ) {
        std::cerr << "! WARNING: Area of two data sets does not match."
                  << std::endl;
    }

    Delta = 0.15 / sqrt(MeanDensity);  //  width of Epanechnikov kernel.

    std::cout << "Delta\t" << Delta << std::endl;

    return true;
}

//////////////////////////////
//

bool  PlotArea::calc_cor(double r_unit, double r_upto)
{
    if (r_unit <= 0 || r_unit > r_upto) {  //  Invalid r specification.
        std::cerr << "Invalid r specification." << std::endl;
        return false;
    }

    R_Unit = r_unit;
    R_Upto = r_upto;

    //  R is kept shorter than the width of the area.

    double min_side = (AreaSize.X < AreaSize.Y) ? AreaSize.X : AreaSize.Y;
    if (R_Upto > min_side) {
        R_Upto = min_side;
    }

    //  r = R_Unit, 2 * R_Unit, 3 * R_Unit, ...(MaxR_Index - 1) * R_Unit.

    MaxR_Index = int (R_Upto / R_Unit);

    initArray();  //  Initialize array to store the results.

    PointList::iterator itr1, itr2;
    PointList::iterator itr2_begin, itr2_end;

    if (Points2.size() > 0) {  //  between two sets of points.
        itr2_begin = Points2.begin();
        itr2_end   = Points2.end();
    }
    else {                     //  within a set of points.
        itr2_begin = Points1.begin();
        itr2_end   = Points1.end();
    }

    for (itr1 = Points1.begin(); itr1 != Points1.end(); ++itr1) {
        for (itr2 = itr2_begin; itr2 != itr2_end; ++itr2) {

            if (itr1 == itr2) {
                continue;
            }

            double r = (*itr1).distance(*itr2);

            for (int i_r = 0; i_r < MaxR_Index; ++i_r) {

                double d = (i_r + 0.5) * R_Unit;
                double w = get_w(r - d);  //  Epanechnikov kernel

                if (w > 0) {
                    double s = get_s(d);  // Ohser's edge correction factor.
                    ResultsArray[i_r].second += w / s / d;
                }
            }
        }
    }

    //  Normalize for point density.

    for (int i = 0; i < MaxR_Index; ++i) {
        ResultsArray[i].second /= (MeanDensity * MeanDensity * 2.0 * M_PI);
    }

    return true;
}

//////////////////////////////
//

const r_X_Array& PlotArea::getResults(void)
{
    return ResultsArray;
}

//////////////////////////////
//
//  Initialize the result array.

void PlotArea::initArray(void)
{
    ResultsArray.erase(ResultsArray.begin(), ResultsArray.end());

    for (int i = 0; i < MaxR_Index; ++i) {
        double r = (i + 0.5) * R_Unit;
        ResultsArray.push_back(r_X_Pair(r, 0.0));
    }
}


//////////////////////////////
//
//   Ohser's edge correction

double PlotArea::get_s(double x)
{
    return Area - x * (Perimeter - x) / M_PI;
}

//////////////////////////////
//
//  Epanechnikov kernel

double PlotArea::get_w(double d)
{
    if (d < 0) {
        d = -d;
    }

    if (d >= Delta) {
        return 0;
    }
    else {
        return  0.75 / Delta * (1.0 - d * d / (Delta * Delta));
    }
}
